from typing import List

from cloudevents.abstract import CloudEvent
from pydantic import BaseModel, PrivateAttr

from venty.entity import Entity as _Entity, EntityId
from venty.strong_types import StreamVersion, NO_EVENT_VERSION


class Entity(BaseModel, _Entity):
    _changes: List[CloudEvent] = PrivateAttr(default_factory=list)
    version: StreamVersion = NO_EVENT_VERSION

    @property
    def entity_changes(self) -> List[CloudEvent]:
        return self._changes

    def clear_entity_changes(self) -> None:
        self._changes = []

    @property
    def entity_version(self) -> StreamVersion:
        return self.version

    def set_entity_version(self, version: StreamVersion) -> None:
        self.version = version

    @property
    def entity_id(self) -> EntityId:
        raise NotImplementedError()

    def when(self, event: CloudEvent) -> None:
        raise NotImplementedError()
